using BepInEx;
using R2API;
using R2API.Utils;
using RoR2;
using UnityEngine;

namespace ExamplePlugin
{
    [BepInDependency(R2API.R2API.PluginGUID)]
	
    [BepInPlugin(PluginGUID, PluginName, PluginVersion)]
	
    [R2APISubmoduleDependency(nameof(ItemAPI), nameof(ItemDropAPI), nameof(LanguageAPI))]
	
    public class BloodStaff : BaseUnityPlugin
	{
        public const string PluginGUID = PluginAuthor + "." + PluginName;
        public const string PluginAuthor = "AuthorName";
        public const string PluginName = "ExamplePlugin";
        public const string PluginVersion = "1.0.0";

        private static ItemDef myItemDef;

        public void Awake()
        {
            Log.Init(Logger);

            myItemDef = ScriptableObject.CreateInstance<ItemDef>();

            myItemDef.name = "EXAMPLE_HEALONHIT_NAME";
            myItemDef.nameToken = "EXAMPLE_HEALONHIT_NAME";
            myItemDef.pickupToken = "EXAMPLE_HEALONHIT_PICKUP";
            myItemDef.descriptionToken = "EXAMPLE_HEALONHIT_DESC";
            myItemDef.loreToken = "EXAMPLE_HEALONHIT_LORE";

            myItemDef.tier = ItemTier.Tier2;

            myItemDef.pickupIconSprite = Resources.Load<Sprite>("Textures/MiscIcons/texMysteryIcon");
            myItemDef.pickupModelPrefab = Resources.Load<GameObject>("Prefabs/PickupModels/PickupMystery");

            myItemDef.canRemove = true;

            myItemDef.hidden = false;
			
            AddTokens();

            var displayRules = new ItemDisplayRuleDict(null);

            ItemAPI.Add(new CustomItem(myItemDef, displayRules));

            GlobalEventManager.onServerDamageDealt += GlobalEventManager_onDamageDealtGlobal;

            Log.LogInfo(nameof(Awake) + " done.");
        }

        private void GlobalEventManager_onDamageDealtGlobal(DamageReport report)
        {
            if (!report.attacker || !report.attackerBody )
                return;
            
            CharacterBody attacker = report.attackerBody;

            if (attacker.inventory)
            {
                int garbCount = attacker.inventory.GetItemCount(myItemDef.itemIndex);
                if (garbCount > 0)
                {
                    float healAmount = report.damageDealt / 50;
                    if (report.combinedHealthBeforeDamage < healAmount)
                    {
                        healAmount = report.combinedHealthBeforeDamage;
                    }

                    attacker.healthComponent.Heal(healAmount * garbCount, default, true);
                }
            }
        }

        private void AddTokens()
        {
            LanguageAPI.Add("EXAMPLE_HEALONHIT_NAME", "Blood Staff");

            LanguageAPI.Add("EXAMPLE_HEALONHIT_PICKUP", "Heal whenever you deal damage");

            LanguageAPI.Add("EXAMPLE_HEALONHIT_DESC", "Whenever you <style=cIsDamage>deal damage</style>, restore health equal to <style=cIsHealing>2% of damage dealt</style> <style=cStack>(+2% per stack)</style>.");
            
            LanguageAPI.Add("EXAMPLE_HEALONHIT_LORE", "The best defense is a good offense.");
        }
    }
}
